@@ -19,6 +19,7 @@ use swc_ecma_visit::{
 
 use self::{
     class_name_tdz::ClassNameTdzFolder,
+    member_init::{MemberInit, MemberInitRecord, PrivAccessor, PrivMethod, PrivProp, PubProp},
     private_field::{
         dup_private_method, visit_private_in_expr, BrandCheckHandler, Private,
         PrivateAccessVisitor, PrivateKind, PrivateRecord,
@@ -28,6 +29,7 @@ use self::{
 };
 
 mod class_name_tdz;
+mod member_init;
 mod private_field;
 mod this_in_static;
 mod used_name;
@@ -41,18 +43,21 @@ mod used_name;
 /// We use custom helper to handle export default class
 pub fn class_properties(config: Config) -> impl Fold + VisitMut {
     as_folder(ClassProperties {
-        config,
+        c: config,
         private: PrivateRecord::new(),
     })
 }
 
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Clone, Copy, Default)]
 pub struct Config {
-    pub loose: bool,
+    pub private_as_properties: bool,
+    pub set_public_fields: bool,
+    pub constant_super: bool,
+    pub no_document_all: bool,
 }
 
 struct ClassProperties {
-    config: Config,
+    c: Config,
     private: PrivateRecord,
 }
 
@@ -384,20 +389,18 @@ impl ClassProperties {
 
         let has_super = class.super_class.is_some();
 
-        let mut typescript_constructor_properties = vec![];
-
-        let mut constructor_exprs = vec![];
+        let mut constructor_inits = MemberInitRecord::new(self.c);
         let mut vars = vec![];
-        let mut extra_stmts = vec![];
+        let mut extra_inits = MemberInitRecord::new(self.c);
         let mut private_method_fn_decls = vec![];
         let mut members = vec![];
         let mut constructor = None;
         let mut used_names = vec![];
         let mut used_key_names = vec![];
-        let mut names_used_for_brand_checks = AHashSet::default();
+        let mut super_ident = None;
 
         class.body.visit_mut_with(&mut BrandCheckHandler {
-            names: &mut names_used_for_brand_checks,
+            names: &mut AHashSet::default(),
             private: &self.private,
         });
 
@@ -412,7 +415,7 @@ impl ClassProperties {
                             span: c_span,
                             mut expr,
                         }) => {
-                            vars.extend(visit_private_in_expr(&mut expr, &self.private));
+                            vars.extend(visit_private_in_expr(&mut expr, &self.private, self.c));
 
                             expr.visit_mut_with(&mut ClassNameTdzFolder {
                                 class_name: &class_ident,
@@ -453,51 +456,58 @@ impl ClassProperties {
                         });
                     }
 
-                    let key = match prop.key {
-                        PropName::Ident(i) => Box::new(Expr::from(Lit::Str(Str {
-                            span: i.span,
-                            value: i.sym,
-                            has_escape: false,
-                            kind: StrKind::Normal {
-                                contains_quote: false,
-                            },
-                        }))),
-                        PropName::Num(num) => Box::new(Expr::from(num)),
-                        PropName::Str(str) => Box::new(Expr::from(str)),
-                        PropName::BigInt(big_int) => Box::new(Expr::from(big_int)),
-
-                        PropName::Computed(mut key) => {
-                            vars.extend(visit_private_in_expr(&mut key.expr, &self.private));
-                            let (ident, aliased) = if let Expr::Ident(i) = &*key.expr {
-                                if used_key_names.contains(&i.sym) {
-                                    (alias_ident_for(&key.expr, "_ref"), true)
-                                } else {
-                                    alias_if_required(&key.expr, "_ref")
-                                }
+                    if let PropName::Computed(key) = &mut prop.key {
+                        vars.extend(visit_private_in_expr(&mut key.expr, &self.private, self.c));
+                        let (ident, aliased) = if let Expr::Ident(i) = &*key.expr {
+                            if used_key_names.contains(&i.sym) {
+                                (alias_ident_for(&key.expr, "_ref"), true)
                             } else {
                                 alias_if_required(&key.expr, "_ref")
-                            };
-                            // ident.span = ident.span.apply_mark(Mark::fresh(Mark::root()));
-                            if aliased {
-                                // Handle computed property
-                                vars.push(VarDeclarator {
-                                    span: DUMMY_SP,
-                                    name: ident.clone().into(),
-                                    init: Some(key.expr),
-                                    definite: false,
-                                });
                             }
-                            Box::new(Expr::from(ident))
+                        } else {
+                            alias_if_required(&key.expr, "_ref")
+                        };
+                        // ident.span = ident.span.apply_mark(Mark::fresh(Mark::root()));
+                        if aliased {
+                            // Handle computed property
+                            vars.push(VarDeclarator {
+                                span: DUMMY_SP,
+                                name: ident.clone().into(),
+                                init: Some(key.expr.take()),
+                                definite: false,
+                            });
                         }
+                        *key.expr = Expr::from(ident);
                     };
 
                     let mut value = prop.value.unwrap_or_else(|| undefined(prop_span));
 
                     value.visit_mut_with(&mut NewTargetInProp);
 
-                    vars.extend(visit_private_in_expr(&mut value, &self.private));
+                    vars.extend(visit_private_in_expr(&mut value, &self.private, self.c));
 
                     if prop.is_static {
+                        if let (Some(super_class), None) = (&mut class.super_class, &super_ident) {
+                            let (ident, aliased) = alias_if_required(&*super_class, "_ref");
+                            super_ident = Some(ident.clone());
+
+                            if aliased {
+                                vars.push(VarDeclarator {
+                                    span: DUMMY_SP,
+                                    name: ident.clone().into(),
+                                    init: None,
+                                    definite: false,
+                                });
+                                let span = super_class.span();
+                                **super_class = Expr::Assign(AssignExpr {
+                                    span,
+                                    op: op!("="),
+                                    left: PatOrExpr::Pat(Box::new(ident.into())),
+                                    right: super_class.take(),
+                                })
+                            }
+                        }
+
                         value.visit_mut_with(&mut SuperFieldAccessFolder {
                             class_name: &class_ident,
                             vars: &mut vars,
@@ -507,42 +517,23 @@ impl ClassProperties {
                             in_injected_define_property_call: false,
                             in_nested_scope: false,
                             this_alias_mark: None,
-                            // TODO: add loose mode
-                            constant_super: false,
-                            super_class: &None,
+                            constant_super: self.c.constant_super,
+                            super_class: &super_ident,
                         });
                         value.visit_mut_with(&mut ThisInStaticFolder {
                             ident: class_ident.clone(),
                         });
                     }
 
-                    let callee = helper!(define_property, "defineProperty");
-
+                    let init = MemberInit::PubProp(PubProp {
+                        span: prop_span,
+                        name: prop.key,
+                        value,
+                    });
                     if prop.is_static {
-                        extra_stmts.push(
-                            CallExpr {
-                                span: DUMMY_SP,
-                                callee,
-                                args: vec![
-                                    class_ident.clone().as_arg(),
-                                    key.as_arg(),
-                                    value.as_arg(),
-                                ],
-                                type_args: Default::default(),
-                            }
-                            .into_stmt(),
-                        )
+                        extra_inits.push(init);
                     } else {
-                        constructor_exprs.push(Box::new(Expr::Call(CallExpr {
-                            span: DUMMY_SP,
-                            callee,
-                            args: vec![
-                                ThisExpr { span: DUMMY_SP }.as_arg(),
-                                key.as_arg(),
-                                value.as_arg(),
-                            ],
-                            type_args: Default::default(),
-                        })));
+                        constructor_inits.push(init);
                     }
                 }
                 ClassMember::PrivateProp(mut prop) => {
@@ -551,12 +542,12 @@ impl ClassProperties {
                     let ident = Ident::new(
                         format!("_{}", prop.key.id.sym).into(),
                         // We use `self.mark` for private variables.
-                        prop.key.span.apply_mark(self.private.curr_mark()),
+                        prop.key.span.apply_mark(self.private.cur_mark()),
                     );
 
                     if let Some(value) = &mut prop.value {
                         value.visit_mut_with(&mut NewTargetInProp);
-                        vars.extend(visit_private_in_expr(&mut *value, &self.private));
+                        vars.extend(visit_private_in_expr(&mut *value, &self.private, self.c));
                     }
 
                     prop.value.visit_with(&mut UsedNameCollector {
@@ -570,75 +561,44 @@ impl ClassProperties {
 
                     let value = prop.value.unwrap_or_else(|| undefined(prop_span));
 
-                    let extra_init = if prop.is_static {
-                        Box::new(Expr::Object(ObjectLit {
-                            span: DUMMY_SP,
-                            props: vec![
-                                PropOrSpread::Prop(Box::new(Prop::KeyValue(KeyValueProp {
-                                    key: PropName::Ident(quote_ident!("writable")),
-                                    value: Box::new(Expr::Lit(Lit::Bool(Bool {
-                                        span: DUMMY_SP,
-                                        value: true,
-                                    }))),
-                                }))),
-                                PropOrSpread::Prop(Box::new(Prop::KeyValue(KeyValueProp {
-                                    key: PropName::Ident(quote_ident!("value")),
-                                    value,
-                                }))),
-                            ],
-                        }))
-                    } else {
-                        constructor_exprs.push(Box::new(Expr::Call(CallExpr {
-                            span: DUMMY_SP,
-                            callee: ident.clone().make_member(quote_ident!("set")).as_callee(),
-                            args: vec![
-                                ThisExpr { span: DUMMY_SP }.as_arg(),
-                                ObjectLit {
-                                    span: DUMMY_SP,
-                                    props: vec![
-                                        // writeable: true
-                                        PropOrSpread::Prop(Box::new(Prop::KeyValue(
-                                            KeyValueProp {
-                                                key: PropName::Ident(quote_ident!("writable")),
-                                                value: Box::new(Expr::Lit(Lit::Bool(Bool {
-                                                    value: true,
-                                                    span: DUMMY_SP,
-                                                }))),
-                                            },
-                                        ))),
-                                        // value: value,
-                                        PropOrSpread::Prop(Box::new(Prop::KeyValue(
-                                            KeyValueProp {
-                                                key: PropName::Ident(quote_ident!("value")),
-                                                value,
-                                            },
-                                        ))),
-                                    ],
-                                }
-                                .as_arg(),
-                            ],
-                            type_args: Default::default(),
-                        })));
-
-                        Box::new(Expr::New(NewExpr {
-                            span: DUMMY_SP,
-                            callee: Box::new(Expr::Ident(quote_ident!("WeakMap"))),
-                            args: Some(vec![]),
-                            type_args: Default::default(),
-                        }))
-                    };
-
-                    extra_stmts.push(Stmt::Decl(Decl::Var(VarDecl {
-                        span: DUMMY_SP,
-                        kind: VarDeclKind::Var,
-                        declare: false,
-                        decls: vec![VarDeclarator {
+                    let init = MemberInit::PrivProp(PrivProp {
+                        span: prop_span,
+                        name: ident.clone(),
+                        value,
+                    });
+                    if self.c.private_as_properties {
+                        vars.push(VarDeclarator {
                             span: DUMMY_SP,
                             definite: false,
                             name: ident.clone().into(),
-                            init: Some(extra_init),
-                        }],
-                    })));
+                            init: Some(Box::new(Expr::from(CallExpr {
+                                span: DUMMY_SP,
+                                callee: helper!(
+                                    class_private_field_loose_key,
+                                    "classPrivateFieldLooseKey"
+                                ),
+                                args: vec![ident.sym.as_arg()],
+                                type_args: Default::default(),
+                            }))),
+                        });
+                    } else if !prop.is_static {
+                        vars.push(VarDeclarator {
+                            span: DUMMY_SP,
+                            definite: false,
+                            name: ident.into(),
+                            init: Some(Box::new(Expr::from(NewExpr {
+                                span: DUMMY_SP,
+                                callee: Box::new(Expr::Ident(quote_ident!("WeakMap"))),
+                                args: Some(Default::default()),
+                                type_args: Default::default(),
+                            }))),
+                        });
+                    };
+                    if prop.is_static {
+                        extra_inits.push(init);
+                    } else {
+                        constructor_inits.push(init);
+                    };
                 }
 
                 ClassMember::Constructor(c) => {
@@ -648,98 +608,128 @@ impl ClassProperties {
                 ClassMember::PrivateMethod(method) => {
                     let is_static = method.is_static;
                     let prop_span = method.span;
+
                     let fn_name = Ident::new(
-                        method.key.id.sym.clone(),
+                        match method.kind {
+                            MethodKind::Getter => format!("get_{}", method.key.id.sym).into(),
+                            MethodKind::Setter => format!("set_{}", method.key.id.sym).into(),
+                            MethodKind::Method => method.key.id.sym.clone(),
+                        },
                         method
                             .span
                             .with_ctxt(SyntaxContext::empty())
-                            .apply_mark(self.private.curr_mark()),
+                            .apply_mark(self.private.cur_mark()),
                     );
 
-                    let should_use_map =
-                        matches!(method.kind, MethodKind::Getter | MethodKind::Setter)
-                            && names_used_for_brand_checks.contains(&method.key.id.sym)
-                            && !is_static;
-
                     let weak_coll_var = Ident::new(
                         format!("_{}", method.key.id.sym).into(),
                         // We use `self.mark` for private variables.
-                        method.key.span.apply_mark(self.private.curr_mark()),
+                        method.key.span.apply_mark(self.private.cur_mark()),
                     );
                     method.function.visit_with(&mut UsedNameCollector {
                         used_names: &mut used_names,
                     });
 
-                    if should_use_map || !is_static {
+                    let extra_collect = match (method.kind, is_static) {
+                        (MethodKind::Getter | MethodKind::Setter, false) => {
+                            let is_getter = method.kind == MethodKind::Getter;
+                            let inserted =
+                                constructor_inits.push(MemberInit::PrivAccessor(PrivAccessor {
+                                    span: prop_span,
+                                    name: weak_coll_var.clone(),
+                                    getter: if is_getter {
+                                        Some(fn_name.clone())
+                                    } else {
+                                        None
+                                    },
+                                    setter: if !is_getter {
+                                        Some(fn_name.clone())
+                                    } else {
+                                        None
+                                    },
+                                }));
+
+                            if inserted {
+                                Some(quote_ident!("WeakMap"))
+                            } else {
+                                None
+                            }
+                        }
+                        (MethodKind::Getter | MethodKind::Setter, true) => {
+                            let is_getter = method.kind == MethodKind::Getter;
+                            let inserted =
+                                extra_inits.push(MemberInit::PrivAccessor(PrivAccessor {
+                                    span: prop_span,
+                                    name: weak_coll_var.clone(),
+                                    getter: if is_getter {
+                                        Some(fn_name.clone())
+                                    } else {
+                                        None
+                                    },
+                                    setter: if !is_getter {
+                                        Some(fn_name.clone())
+                                    } else {
+                                        None
+                                    },
+                                }));
+                            if inserted && self.c.private_as_properties {
+                                Some(Ident::dummy())
+                            } else {
+                                None
+                            }
+                        }
+
+                        (MethodKind::Method, false) => {
+                            constructor_inits.push(MemberInit::PrivMethod(PrivMethod {
+                                span: prop_span,
+                                name: weak_coll_var.clone(),
+                                fn_name: if self.c.private_as_properties {
+                                    fn_name.clone()
+                                } else {
+                                    Ident::dummy()
+                                },
+                            }));
+                            Some(quote_ident!("WeakSet"))
+                        }
+                        (MethodKind::Method, true) => {
+                            if self.c.private_as_properties {
+                                extra_inits.push(MemberInit::PrivMethod(PrivMethod {
+                                    span: prop_span,
+                                    name: weak_coll_var.clone(),
+                                    fn_name: fn_name.clone(),
+                                }));
+                                Some(Ident::dummy())
+                            } else {
+                                None
+                            }
+                        }
+                    };
+
+                    if let Some(extra) = extra_collect {
                         vars.push(VarDeclarator {
                             span: DUMMY_SP,
                             definite: false,
                             name: weak_coll_var.clone().into(),
-                            init: Some(Box::new(Expr::from(NewExpr {
-                                span: DUMMY_SP,
-                                callee: if should_use_map {
-                                    Box::new(Expr::Ident(quote_ident!("WeakMap")))
-                                } else {
-                                    Box::new(Expr::Ident(quote_ident!("WeakSet")))
-                                },
-                                args: Some(Default::default()),
-                                type_args: Default::default(),
-                            }))),
-                        });
-                    }
-
-                    if should_use_map {
-                        let get = PropOrSpread::Prop(Box::new(Prop::KeyValue(KeyValueProp {
-                            key: PropName::Ident(quote_ident!("get")),
-                            value: Box::new(Expr::Ident(fn_name.clone())),
-                        })));
-                        let set = PropOrSpread::Prop(Box::new(Prop::KeyValue(KeyValueProp {
-                            key: PropName::Ident(quote_ident!("set")),
-                            value: undefined(DUMMY_SP),
-                        })));
-
-                        let obj = ObjectLit {
-                            span: DUMMY_SP,
-                            props: vec![get, set],
-                        };
-
-                        let obj = if is_static {
-                            let var_name = private_ident!("static_method");
-
-                            vars.push(VarDeclarator {
-                                span: DUMMY_SP,
-                                name: var_name.clone().into(),
-                                init: Some(Box::new(Expr::Object(obj))),
-                                definite: Default::default(),
-                            });
-
-                            var_name.as_arg()
-                        } else {
-                            obj.as_arg()
-                        };
-
-                        constructor_exprs.push(Box::new(Expr::Call(CallExpr {
-                            span: prop_span,
-                            callee: weak_coll_var
-                                .clone()
-                                .make_member(quote_ident!("set"))
-                                .as_callee(),
-                            args: vec![ThisExpr { span: DUMMY_SP }.as_arg(), obj],
-                            type_args: Default::default(),
-                        })));
-                    } else if !is_static {
-                        // Add `_get.add(this);` to the constructor where `_get` is the name of the
-                        // weak set.
-                        constructor_exprs.push(Box::new(Expr::Call(CallExpr {
-                            span: prop_span,
-                            callee: weak_coll_var
-                                .clone()
-                                .make_member(quote_ident!("add"))
-                                .as_callee(),
-                            args: vec![ThisExpr { span: DUMMY_SP }.as_arg()],
-                            type_args: Default::default(),
-                        })));
-                    }
+                            init: Some(Box::new(if self.c.private_as_properties {
+                                Expr::from(CallExpr {
+                                    span: DUMMY_SP,
+                                    callee: helper!(
+                                        class_private_field_loose_key,
+                                        "classPrivateFieldLooseKey"
+                                    ),
+                                    args: vec![weak_coll_var.sym.as_arg()],
+                                    type_args: Default::default(),
+                                })
+                            } else {
+                                Expr::New(NewExpr {
+                                    span: DUMMY_SP,
+                                    callee: Box::new(Expr::Ident(extra)),
+                                    args: Some(Default::default()),
+                                    type_args: Default::default(),
+                                })
+                            })),
+                        })
+                    };
 
                     private_method_fn_decls.push(Stmt::Decl(Decl::Fn(FnDecl {
                         ident: fn_name,
@@ -754,12 +744,7 @@ impl ClassProperties {
             }
         }
 
-        let constructor_exprs = {
-            typescript_constructor_properties.extend(constructor_exprs);
-            typescript_constructor_properties
-        };
-
-        let constructor = self.process_constructor(constructor, has_super, constructor_exprs);
+        let constructor = self.process_constructor(constructor, has_super, constructor_inits);
         if let Some(c) = constructor {
             members.push(ClassMember::Constructor(c));
         }
@@ -768,14 +753,18 @@ impl ClassProperties {
             private: &self.private,
             vars: vec![],
             in_assign_pat: false,
+            c: self.c,
         });
 
+        let mut extra_stmts = extra_inits.into_init_static(class_ident.clone());
+
         extra_stmts.extend(private_method_fn_decls);
 
         members.visit_mut_with(&mut PrivateAccessVisitor {
             private: &self.private,
             vars: vec![],
             in_assign_pat: false,
+            c: self.c,
         });
 
         self.private.pop();
@@ -828,10 +817,10 @@ impl ClassProperties {
         &mut self,
         constructor: Option<Constructor>,
         has_super: bool,
-        constructor_exprs: Vec<Box<Expr>>,
+        constructor_exprs: MemberInitRecord,
     ) -> Option<Constructor> {
         let constructor = constructor.or_else(|| {
-            if constructor_exprs.is_empty() {
+            if constructor_exprs.record.is_empty() {
                 None
             } else {
                 Some(default_constructor(has_super))
@@ -839,6 +828,7 @@ impl ClassProperties {
         });
 
         if let Some(mut c) = constructor {
+            let constructor_exprs = constructor_exprs.into_init();
             // Prepend properties
             inject_after_super(&mut c, constructor_exprs);
             Some(c)
